import 'dart:async';
import 'dart:io';

int fib(int v) {
  if (v <= 1) return v;
  return fib(v - 1) + fib(v - 2);
}

/**
 * Splits the incoming string into lines. Emits a new data event for each line.
 */
class LineTransformer extends StreamTransformer {
  String _buffer = "";

  void handleData(String data, Sink sink) {
    int index = data.indexOf('\n');
    if (index >= 0) {
      int start = 0;
      do {
        sink.add("$_buffer${data.substring(start, index)}");
        _buffer = "";
        start = index + 1;
        index = data.indexOf('\n', start);
      } while (index >= start);
      _buffer = data.substring(start);
    } else {
      _buffer = "$_buffer$data";
    }
  }

  void handleDone(Sink sink) {
    if (_buffer != "") sink.add(_buffer);
    sink.close();
  }
}

void main() {
  new File('numbers.dat').openForRead()
    .transform(new Utf8Decoder())
    .transform(new LineTransformer())
    .mappedBy(int.parse)
    .mappedBy(fib)
    .subscribe(onData: print);
}
